@@ -53,6 +53,20 @@ import click
 
 import rasterio
 import rasterio.shutil
+from rasterio.dtypes import (
+    ubyte,
+    uint8,
+    uint16,
+    uint32,
+    uint64,
+    int8,
+    int16,
+    int32,
+    int64,
+    float16,
+    float32,
+    float64
+)
 from rasterio._path import _parse_path, _UnparsedPath
 
 
@@ -275,8 +289,20 @@ dimensions_opt = click.option(
 dtype_opt = click.option(
     '-t', '--dtype',
     type=click.Choice([
-        'ubyte', 'uint8', 'uint16', 'int16', 'uint32', 'int32',
-        'float32', 'float64']),
+        ubyte,
+        uint8,
+        uint16,
+        uint32,
+        uint64,
+        int8,
+        int16,
+        uint32,
+        int32,
+        int64,
+        float16,
+        float32,
+        float64
+    ]),
     default=None,
     help="Output data type.")
 
@@ -319,7 +345,6 @@ creation_options = click.option(
 rgb_opt = click.option(
     '--rgb', 'photometric',
     flag_value='rgb',
-    default=False,
     help="Set RGB photometric interpretation.")
 
 overwrite_opt = click.option(
@@ -370,3 +395,47 @@ sequence_opt = click.option(
 format_opt = click.option(
     "-f", "--format", "--driver", "driver", help="Output format driver."
 )
+
+
+def geojson_type_opt(*, allowed, default):
+    """GeoJSON output mode"""
+    def verify_geojson_type(ctx, param, value):
+        geojson_type = ctx.params.setdefault("geojson_type", {})
+        geojson_type[param.name] = value
+        if len(geojson_type) == len(allowed):
+            # We've now seen all options for geojson_type, so verify their contents.
+            specified = [
+                name.removeprefix("geojson_type_")
+                for name, value in geojson_type.items()
+                if value
+            ]
+            if len(specified) > 1:
+                names = ",".join(f"--{name}" for name in specified)
+                raise click.BadParameter(
+                    f"too many GeoJSON output formats: {names} may not be specified together"
+                )
+            ctx.params["geojson_type"] = specified[0] if specified else default
+
+    def wrapper(func):
+        options = {
+            "collection": "feature collection(s)",
+            "feature": "feature(s)",
+            "bbox": "bounding box array(s)",
+        }
+
+        for name in allowed:
+            if name not in options:
+                raise click.UsageError(f"{name} is not a valid GeoJSON output mode")
+            help_suffix = " (default)" if name == default else ""
+            opt = click.option(
+                f"--{name}",
+                f"geojson_type_{name}",
+                is_flag=True,
+                expose_value=False,
+                callback=verify_geojson_type,
+                help=f"Output as GeoJSON {options[name]}{help_suffix}",
+            )
+            func = opt(func)
+        return func
+
+    return wrapper
